==> flourish-0.4.2.tar.gz: flourish-0.4.2/flourish/url.py <==
from datetime import datetime
import re


class URL(object):
    def __init__(self, parent, path, name, generator):
        self.parent = parent
        self.path = path
        self.name = name
        self.generator = generator

    @property
    def arguments(self):
        _arguments = []
        for _segment in re.split('(#\w+)', self.path):
            if _segment.startswith('#'):
                _arguments.append(_segment[1:])
        return _arguments

    def resolve(self, **kwargs):
        _resolved = ''
        for _segment in re.split('(#\w+)', self.path):
            if _segment.startswith('#'):
                key = _segment[1:]
                if key in kwargs:
                    if kwargs[key] is None:
                        raise RuntimeError
                    _resolved = _resolved + kwargs[key]
                else:
                    raise KeyError
            else:
                _resolved = _resolved + _segment
        return _resolved

    def all_valid_filters(self):
        _valid_filters = []
        _args = self.arguments

        if len(_args) == 0:
            _valid_filters.append({})
        else:
            _filters = self.get_all_filters_for_arguments(_args)
            for _dict in _filters:
                _valid_filters.append(_dict)
        return _valid_filters

    def get_all_filters_for_arguments(self, keys, objects=None):
        # FIXME this should not be a URL method, it doesn't depend on
        # any specific URL
        if objects is None:
            objects = self.parent

        _first_key = keys[0]

        _values = set()
        for _source in objects:
            _is_date_filter = (
                _first_key in ['year', 'month', 'day'] and
                'published' in _source and
                type(_source['published'] == datetime)
            )

            if _is_date_filter:
                _value = getattr(_source['published'], _first_key)
                _values.add('%02d' % _value)
                # _values.add(getattr(_source['published'], _first_key))
                # if _first_key == 'year':
                #     _values.add('dateyear', _first_key)
                # elif _first_key == 'month':
                #     _values.add('datemonth', '%02s' % _first_key)
                # else:
                #     _values.add('dateday', '%02s' % _first_key)
            elif _first_key in _source:
                if type(_source[_first_key]) == list:
                    for _value in _source[_first_key]:
                        _values.add(_value)
                else:
                    _values.add(_source[_first_key])

        _filters = []
        for _value in _values:
            _dict = {_first_key: _value}
            if len(keys) == 1:
                _filters.append(_dict)
            else:
                _sub_objects = objects.filter(**_dict)
                _sub_keys = self.get_all_filters_for_arguments(
                    keys[1:], _sub_objects)
                for _sub_key in _sub_keys:
                    _update = _dict.copy()
                    _update.update(_sub_key)
                    _filters.append(_update)

        # the sort is only so the tests can compare results easily
        return sorted(_filters)

    def __repr__(self):
        return '<flourish.URL object (%s: %s)>' % (self.name, self.path)

==> flourish-0.7.0.tar.gz: flourish-0.7.0/flourish/url.py <==
import re


class URL(object):
    def __init__(self, parent, path, name, generator):
        self.parent = parent
        self.path = path
        self.name = name
        self.generator = generator

    @property
    def arguments(self):
        _arguments = []
        for _segment in re.split(r'(#\w+)', self.path):
            if _segment.startswith('#'):
                _arguments.append(_segment[1:])
        return _arguments

    def resolve(self, **kwargs):
        _resolved = ''
        for _segment in re.split(r'(#\w+)', self.path):
            if _segment.startswith('#'):
                key = _segment[1:]
                if key in kwargs:
                    if kwargs[key] is None:
                        raise RuntimeError
                    _resolved = _resolved + kwargs[key]
                else:
                    raise KeyError
            else:
                _resolved = _resolved + _segment
        return _resolved

    def can_generate(self, path):
        subpath = None
        if path.endswith('?'):
            subpath = path[:len(path)-1]
        generates = []
        for _dict in self.all_valid_filters():
            x = self.resolve(**_dict)
            if path == x:
                generates.append(_dict)
            if subpath and x.startswith(subpath):
                generates.append(_dict)
        return generates

    def all_valid_filters(self):
        _valid_filters = []
        _args = self.arguments

        if len(_args) == 0:
            _valid_filters.append({})
        else:
            _filters = self.parent.copy().get_valid_filters_for_tokens(_args)
            for _dict in _filters:
                _valid_filters.append(_dict)
        return _valid_filters

    def __repr__(self):
        return '<flourish.URL object (%s: %s)>' % (self.name, self.path)

    class DoesNotExist(Exception):
        pass
